//*          LCD Codelock              *
// 
// Compile with free Hi Tech that comes with MPLAB 8.63 or lower. 
// This code is free to use, for more info   http://www.moty22.co.uk       
   
#include <htc.h>

#define _XTAL_FREQ 4000000
#define bk digit(32)
#define code_sw RB7
#define out RA2		//relay drive 
#define rs RA0	//LCD reg
#define e RA1	//LCD clock

//prototypes
void main(void);
void clock(void);
void digit(char fig);
void line(char msg);
void convert(char key);
void convert_mem(char mem_key);
void new_code(void);

__CONFIG(LVPDIS & WDTDIS & UNPROTECT & XT);

unsigned char d[4]={0,1,2,3};
unsigned char mem[4]={0,1,2,3};	//memory array
unsigned char press,write;		//
		
void main(void){
unsigned char read=0, idle=0;

	//init
	T1CON = 0b110000;        //TMR1 prescale 1:8, osc=off, timer off
	ADCON1 = 0b110;			//port-a digital
	TRISB = 0b11111111;			 //all inputs
	TRISA = 0b11111000;		//RA0-RA2 outputs
	TRISC = 0b11110000;
	RBPU = 0;			//RB pullup on
	RB7 = 0;
	

	//LCD init, LCD module is preset for 8 bits data so we need to set it for 4 bits mode.
	__delay_ms(250);
	e=0; rs=0;
	PORTC=2; clock();	//4 bits mode
	PORTC=2; clock();	//function set: 4 bit
	PORTC=0; clock();	//function set: 1 line, 2 lines=8
	__delay_us(25); __delay_us(25);
	PORTC=0; clock();	//display set:
	PORTC=14; clock();	//display set: curzor off, curzor on=14, curzor on and flashing=15
	__delay_us(25); __delay_us(25);
	PORTC=0; clock(); PORTC=1; clock();	//display clear
	__delay_ms(2); 
	PORTC=0; clock();	//Entry mode:
	PORTC=6; clock();	//Entry mode:increment address
	
	while(read < 4){mem[read] = eeprom_read(read);++read;}	//read eeprom

	for(;;){
		start:
		out = 0;
		press=0;
	//	d[0]=42; d[1]=42; d[2]=42; d[3]=42;	//42=asterisk
		line(1);
		while(press < 4){					//loop for entering 4 numbers
			
			while (!RBIF){					//are inuts changed?
			if(!code_sw){new_code(); goto start;}	//
			TMR1ON=1;
			if(TMR1IF){++idle; TMR1IF=0;}
			if(idle > 60){idle=0; TMR1ON=0; TMR1IF=0; goto start;}	//if keys are idle for 30 seconds restart.
			}
			__delay_ms(20);					//20ms for switch bounce
			PORTB=PORTB;
			RBIF=0;
			
			if(!RB4 || !RB5 || !RB6){
				idle=0;				//reset timer
				d[press]=PORTB & 0x7F; convert(d[press]);digit(d[press]);++press;		//read code, add digit to the line
				}
		}				
		if(d[0]==mem[0] && d[1]==mem[1] && d[2]==mem[2] && d[3]==mem[3]){out=1;line(2);}	//compare code to memory
		else{line(3);}
		__delay_ms(2000);
	
	}
}

void new_code(void){
	write=0;
//	mem[0]=42; mem[1]=42; mem[2]=42; mem[3]=42;	//42=asterisk
	line(4); 
	while(write < 4){
		while (!RBIF){}		//are inuts changed?
		__delay_ms(20);		//20ms
		PORTB=PORTB;
		RBIF=0;
			if(!RB4 || !RB5 || !RB6){
				mem[write]=PORTB & 0x7F;	//remove rb7 from reading
				convert_mem(mem[write]);
				eeprom_write((write),mem[write]);		//write eeprom
				__delay_ms(20);
				digit(mem[write]);		//add digit to the line
				++write;
			}			
	}
}

void clock(void){		//LCD clock
	e=1;
	__delay_us(25);
	e=0;
	__delay_us(25);
}

void line(char msg){			//write a line to the lcd 
	
//clear display
	rs=0;
	PORTC=0; clock(); PORTC=1; clock(); 	
	PORTC=0; clock(); PORTC=2; clock();	//return home
	__delay_ms(2);					//2ms
	
	switch (msg) {
		case 1: bk;bk;bk;
				digit(0x43);		//enter code
				digit(0x4F);
				digit(0x44);
				digit(0x45);
				bk;bk;		//blank
				break;
		case 2: bk;bk;bk;bk;
				digit(0x43);		//code ok
				digit(0x4F);
				digit(0x44);
				digit(0x45);
				bk;bk;
				digit(0x4F);
				digit(0x4B);
				bk;bk;bk;bk;
				break;
		case 3: bk;bk;bk;
				digit(0x57);		//wrong code
				digit(0x52);
				digit(0x4F);
				digit(0x4E);
				digit(0x47);
				digit(32);
				digit(0x43);
				digit(0x4F);
				digit(0x44);
				digit(0x45);
				bk;bk;bk;
				break;
		case 4: bk;digit(0x4E);		//new code
				digit(0x45);
				digit(0x57);
				bk;
				digit(0x43);
				digit(0x4F);
				digit(0x44);
				digit(0x45);
				bk;bk;
				break;				
	}			
}

void digit(char fig){		//write a digit to the lcd

	rs=1;
	PORTC=(fig >> 4); clock();			//send 4 MSB
	PORTC=fig & 0b00001111; clock();	//send 4 LSB
	rs=0;

}

//convert buttons to LCD character	
void convert(char key){	
	switch (key) {
		case 0b111110: d[press]=49; break;	//1
		case 0b1011110: d[press]=50; break;   
		case 0b1101110: d[press]=51 ;break; 
		case 0b111101: d[press]=52; break;	//4
		case 0b1011101: d[press]=53; break;   
		case 0b1101101: d[press]=54 ;break; 
		case 0b111011: d[press]=55; break;	//7
		case 0b1011011: d[press]=56; break;   
		case 0b1101011: d[press]=57 ;break; 
		case 0b1010111: d[press]=48; break;	//0
	}	  
}

void convert_mem(char mem_key){
	switch (mem_key) {
		case 0b111110: mem[write]=49; break;	//1
		case 0b1011110: mem[write]=50; break;   
		case 0b1101110: mem[write]=51 ;break; 
		case 0b111101: mem[write]=52; break;	//4
		case 0b1011101: mem[write]=53; break;   
		case 0b1101101: mem[write]=54 ;break; 
		case 0b111011: mem[write]=55; break;	//7
		case 0b1011011: mem[write]=56; break;   
		case 0b1101011: mem[write]=57 ;break; 
		case 0b1010111: mem[write]=48; break;	//0
	}	  
}


